@@ -2,7 +2,7 @@
 //
 // This code was created in 2010 for the Library of Congress and the
 // other federal government agencies participating in the Federal Agencies
-// Digitization Guidelines Initiative and it is in the public domain.
+// Digital Guidelines Initiative and it is in the public domain.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -21,11 +21,10 @@
 #include <QTableWidget>
 #include <QFileDialog>
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QComboBox>
 #include <QTextEdit>
 #include <QMessageBox>
-#include <QDesktopWidget>
+#include <QScreen>
 //---------------------------------------------------------------------------
 
 //***************************************************************************
@@ -41,7 +40,7 @@ CodingHistoryDialog_TableWidget::CodingHistoryDialog_TableWidget (ZtringListList
 }
 
 //---------------------------------------------------------------------------
-void CodingHistoryDialog_TableWidget::dataChanged ( const QModelIndex & topLeft, const QModelIndex & bottomRight, const QVector<int> & roles) 
+void CodingHistoryDialog_TableWidget::dataChanged ( const QModelIndex & topLeft, const QModelIndex&, const QVector<int>&)
 {
     if (Updating)
         return;
@@ -49,16 +48,35 @@ void CodingHistoryDialog_TableWidget::dataChanged ( const QModelIndex & topLeft,
     //Retrieving data
     size_t Row=(size_t)topLeft.row();
     size_t Column=(size_t)topLeft.column();
-    string Value=topLeft.model()->data(topLeft.model()->index(topLeft.row(), topLeft.column(), rootIndex())).toString().toLocal8Bit().data();
-    
+    string Value=topLeft.model()->data(topLeft.model()->index(topLeft.row(), topLeft.column(), rootIndex())).toString().toUtf8().data();
+
+    if (!Value.empty())
+    {
+        Char Header;
+        switch (Column)
+        {
+            case 0 : Header='A'; break;
+            case 1 : Header='F'; break;
+            case 2 : Header='B'; break;
+            case 3 : Header='W'; break;
+            case 4 : Header='M'; break;
+            case 5 : Header='T'; break;
+            default: Header='\0';
+        }
+        if (Header!='\0')
+        {
+            Value.insert(Value.begin(), '=');
+            Value.insert(Value.begin(), Header);
+        }
+    }
     //Sizing
     if (Row>=List->size())
-        List->resize(Row+1);    
+        List->resize(Row+1);
     if (Column>=(*List)[Row].size())
-        (*List)[Row].resize(Column+1);    
+        (*List)[Row].resize(Column+1);
 
     //Filling
-    (*List)[Row][Column]=Value;
+    (*List)[Row][Column]=Ztring().From_UTF8(Value);
 
     //Adapting
     if (Row+1==(*List).size())
@@ -80,15 +98,17 @@ void CodingHistoryDialog_TableWidget::dataChanged ( const QModelIndex & topLeft,
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-CodingHistoryDialog_Delegate::CodingHistoryDialog_Delegate(QObject *parent)
+CodingHistoryDialog_Delegate::CodingHistoryDialog_Delegate( Riff_Handler::rules Rules_, QObject *parent)
 : QItemDelegate(parent)
 {
+    Rules=Rules_;
 }
 
 //---------------------------------------------------------------------------
 QWidget *CodingHistoryDialog_Delegate::createEditor(QWidget *parent, const QStyleOptionViewItem &/* option */, const QModelIndex &/* index */) const
 {
     QComboBox* Editor=new QComboBox(parent);
+    Editor->setEditable(!Rules.CodingHistory_Rec && !Rules.FADGI_Rec);
     Editor->installEventFilter(const_cast<CodingHistoryDialog_Delegate*>(this));
 
     return Editor;
@@ -101,10 +121,17 @@ void CodingHistoryDialog_Delegate::setEditorData(QWidget *editor, const QModelIn
     QComboBox* Editor=static_cast<QComboBox*>(editor);
 
     Fill(Editor);
-    
+
+    bool Found=false;
     for (int Pos=0; Pos<Editor->count(); Pos++)
         if (Value==Editor->itemText(Pos))
+         {
+            Found=true;
             Editor->setCurrentIndex(Pos);
+         }
+
+    if (!Rules.CodingHistory_Rec && !Rules.FADGI_Rec && !Value.isEmpty() && !Found)
+        Editor->setCurrentText(Value);
 }
 
 //---------------------------------------------------------------------------
@@ -122,7 +149,6 @@ void CodingHistoryDialog_Delegate::updateEditorGeometry(QWidget *editor, const Q
     editor->setGeometry(option.rect);
 }
 
-
 //***************************************************************************
 // CodingAlgorithmDelegate
 //***************************************************************************
@@ -132,6 +158,8 @@ void CodingAlgorithmDelegate::Fill(QComboBox* Editor) const
 {
     Editor->addItem("");
     Editor->addItem("ANALOGUE");
+    if (!Rules.CodingHistory_Rec || Rules.CodingHistory_Rec_Ex_Analog)
+        Editor->addItem("ANALOG");
     Editor->addItem("PCM");
     Editor->addItem("MPEG1L1");
     Editor->addItem("MPEG1L2");
@@ -155,9 +183,16 @@ void SamplingFrequencyDelegate::Fill(QComboBox* Editor) const
     Editor->addItem("32000");
     Editor->addItem("44100");
     Editor->addItem("48000");
-    Editor->addItem("64000");
-    Editor->addItem("88200");
-    Editor->addItem("96000");
+    if (!Rules.CodingHistory_Rec || Rules.CodingHistory_Rec_Ex_Frequency)
+    {
+        Editor->addItem("64000");
+        Editor->addItem("88200");
+        Editor->addItem("96000");
+        Editor->addItem("176400");
+        Editor->addItem("192000");
+        Editor->addItem("384000");
+        Editor->addItem("768000");
+    }
 }
 
 //***************************************************************************
@@ -209,7 +244,10 @@ void WordLengthDelegate::Fill(QComboBox* Editor) const
     Editor->addItem("20");
     Editor->addItem("22");
     Editor->addItem("24");
-    Editor->addItem("32");
+    if (!Rules.CodingHistory_Rec || Rules.CodingHistory_Rec_Ex_WordLength)
+    {
+        Editor->addItem("32");
+    }
 }
 
 //***************************************************************************
@@ -224,6 +262,13 @@ void ModeDelegate::Fill(QComboBox* Editor) const
     Editor->addItem("stereo");
     Editor->addItem("dual-mono");
     Editor->addItem("joint-stereo");
+    if (!Rules.CodingHistory_Rec)
+    {
+        Editor->addItem("multitrack");
+        Editor->addItem("multichannel");
+        Editor->addItem("surround");
+        Editor->addItem("other");
+    }
 }
 
 //***************************************************************************
@@ -231,27 +276,28 @@ void ModeDelegate::Fill(QComboBox* Editor) const
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-GUI_Main_xxxx_CodingHistoryDialog::GUI_Main_xxxx_CodingHistoryDialog(Core* C_, const std::string &FileName_, const std::string &Field_, const QString &OldText, bool Rules_Recommendations_, QWidget* parent)
+GUI_Main_xxxx_CodingHistoryDialog::GUI_Main_xxxx_CodingHistoryDialog(Core* C_, const std::string &FileName_, const std::string &Field_, const QString &OldText, Riff_Handler::rules Rules_, QWidget* parent)
 : QDialog(parent)
 {
     //Internal
     C=C_;
     FileName=FileName_;
     Field=Field_;
-    Rules_Recommendations=Rules_Recommendations_;
+    Rules=Rules_;
     IsAccepted=false;
 
     //Configuration
-    setWindowFlags(windowFlags()&(0xFFFFFFFF-Qt::WindowContextHelpButtonHint));
+    setWindowFlags(windowFlags()&(~Qt::WindowContextHelpButtonHint));
     setWindowTitle("Coding History");
-    setWindowIcon (QIcon(":/Image/FADGI/Logo.png"));
+    setWindowIcon (QIcon(":/Image/Logo/Logo.png"));
 
     //Buttons
     Load=new QPushButton("&Load file...");
     Save=new QPushButton("&Save file...");
     QDialogButtonBox* Dialog=new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
     Dialog->addButton(Load, QDialogButtonBox::ResetRole);
-    Dialog->addButton(Save, QDialogButtonBox::ResetRole);
+    if (!FileName.empty())
+        Dialog->addButton(Save, QDialogButtonBox::ResetRole);
     connect(Dialog, SIGNAL(accepted()), this, SLOT(OnAccept()));
     connect(Dialog, SIGNAL(rejected()), this, SLOT(reject()));
     connect(Load, SIGNAL(clicked()), this, SLOT(OnMenu_Load()));
@@ -260,35 +306,57 @@ GUI_Main_xxxx_CodingHistoryDialog::GUI_Main_xxxx_CodingHistoryDialog(Core* C_, c
     //Central
     TextEdit=new QTextEdit(this);
     Table=new CodingHistoryDialog_TableWidget(List, this);
+    AddComma=new QCheckBox("&Add comma at EOL");
+    QWidget* W=new QWidget();
+    QVBoxLayout* E=new QVBoxLayout(W);
+    E->addWidget(Table);
+    E->addWidget(AddComma);
+
     Central=new QTabWidget(this);
-    Central->addTab(Table   , tr("EBU R98-1999"));
-    Central->addTab(TextEdit, tr("Free text"));
+    Central->addTab(W       , tr("&EBU R98-1999"));
+    Central->addTab(TextEdit, tr("&Free text"));
     connect(Central, SIGNAL(currentChanged (int)), this, SLOT(OnCurrentChanged(int)));
-        
+
+    Table->setEditTriggers(QAbstractItemView::AllEditTriggers);
+
     QVBoxLayout* L=new QVBoxLayout();
     L->addWidget(Central);
     L->addWidget(Dialog);
 
     setLayout(L);
 
-    resize(QApplication::desktop()->screenGeometry().width()/2, QApplication::desktop()->screenGeometry().height()/3);
+    QScreen* Screen=QApplication::screenAt(mapToGlobal(QPoint(0,0)));
+    if (Screen)
+        resize(Screen->availableGeometry().width()/2, Screen->availableGeometry().height()/3);
+
+    if (Rules.CodingHistory_Rec && !Rules.CodingHistory_Rec_Ex_Comma)
+        AddComma->setEnabled(false);
 
     //Filling
     TextEdit->setPlainText(OldText);
-    string Text=OldText.toLocal8Bit().data();
+    string Text=OldText.toUtf8().data();
     if (Text.find("Coding History ")==0)
-        Text.erase(0, 15);    
+        Text.erase(0, 15);
     if (Text.size()==0
      || (Text.size()>=2
       && Text[1]=='='
       && (Text[0]=='A' || Text[0]=='F' || Text[0]=='B' || Text[0]=='W' || Text[0]=='M' || Text[0]=='T')
-      && Text.find("ANALOG,")==string::npos
       && Text.find(", ")==string::npos))
         OnCurrentChanged(0); //By default, index=0
     else
         Central->setCurrentIndex(1);
 }
 
+//***************************************************************************
+// Public functions
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+QString GUI_Main_xxxx_CodingHistoryDialog::Value() const
+{
+    return TextEdit->toPlainText();
+}
+
 //***************************************************************************
 // Menu actions
 //***************************************************************************
@@ -297,7 +365,7 @@ GUI_Main_xxxx_CodingHistoryDialog::GUI_Main_xxxx_CodingHistoryDialog(Core* C_, c
 void GUI_Main_xxxx_CodingHistoryDialog::OnAccept ()
 {
     IsAccepted=true;
-    
+
     accept();
 }
 
@@ -305,8 +373,8 @@ void GUI_Main_xxxx_CodingHistoryDialog::OnAccept ()
 void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Load()
 {
     //User interaction
-    QString FileNamesQ = QFileDialog::getOpenFileName(this, "", "", "");
-    
+    QString FileNamesQ = QFileDialog::getOpenFileName(this, "", QString::fromUtf8(C->OpenSaveFolder.c_str()), "");
+
     if (FileNamesQ.isEmpty())
         return;
 
@@ -318,7 +386,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Load()
         return;
 
     //Creating buffer
-    int8u* Buffer=new int8u[(size_t)F_Size+1];
+    int8u* Buffer=new int8u[(size_t)F_Size];
     size_t Buffer_Offset=0;
 
     //Reading the file
@@ -331,14 +399,12 @@ void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Load()
     }
     if (Buffer_Offset<F_Size)
         return;
-    Buffer[Buffer_Offset]='\0';
 
     //Filling
-    Ztring ModifiedContent((const char*)Buffer);
+    string ModifiedContent((const char*)Buffer, Buffer_Offset);
     delete[] Buffer;
-    ModifiedContent.FindAndReplace("\r\n", "\n", 0, Ztring_Recursive);
-    ModifiedContent.FindAndReplace("\r", "\n", 0, Ztring_Recursive);
-    QString ModifiedContentQ=QString().fromUtf8(ModifiedContent.To_Local().c_str());
+    AdaptEOL(ModifiedContent, adapt_n);
+    QString ModifiedContentQ=QString::fromUtf8(ModifiedContent.c_str());
 
     TextEdit->setPlainText(ModifiedContentQ);
     if (Central->currentIndex()==0)
@@ -348,9 +414,12 @@ void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Load()
 //---------------------------------------------------------------------------
 void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Save()
 {
+    if (Central->currentIndex()==0)
+        List2Text();
+
     //User interaction
-    QString FileNamesQ = QFileDialog::getSaveFileName(this, "", "", "");
-    
+    QString FileNamesQ = QFileDialog::getSaveFileName(this, "", QString::fromUtf8(C->OpenSaveFolder.c_str()), "");
+
     if (FileNamesQ.isEmpty())
         return;
 
@@ -359,7 +428,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::OnMenu_Save()
         return;
 
     //Filling
-    F.Write(Ztring(TextEdit->toPlainText().toLocal8Bit().data()));
+    F.Write(Ztring(TextEdit->toPlainText().toUtf8().data()));
 }
 
 //---------------------------------------------------------------------------
@@ -399,36 +468,23 @@ void GUI_Main_xxxx_CodingHistoryDialog::List2Text ()
         {
             if (!List[Line][Column].empty())
             {
-                Char Header;
-                switch (Column)
-                {
-                    case 0 : Header='A'; break;
-                    case 1 : Header='F'; break;
-                    case 2 : Header='B'; break;
-                    case 3 : Header='W'; break;
-                    case 4 : Header='M'; break;
-                    case 5 : Header='T'; break;
-                    default: Header='\0';
-                }
-                if (Header!='\0')
-                {
-                    ToReturn+=Header;
-                    ToReturn+='=';
-                }
                 ToReturn+=List[Line][Column];
                 if (Column!=List[Line].size())
                     ToReturn+=',';
             }
         }
-        
+
         while (!ToReturn.empty() && (*(ToReturn.end()-1))==',')
             ToReturn.erase(ToReturn.end()-1);
-            
+
+        if (AddComma->isChecked())
+            ToReturn+=',';
+
         ToReturn+='\n';
     }
-    
+
     if (!ToReturn.empty())
-        TextEdit->setPlainText(ToReturn.To_Local().c_str());
+        TextEdit->setPlainText(QString::fromUtf8(ToReturn.To_UTF8().c_str()));
 }
 
 //---------------------------------------------------------------------------
@@ -436,23 +492,24 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
 {
     //Preparing
     Table->Updating=true;
-	Table->clearContents();
+    Table->clearContents();
     //Warning display
     bool Ununderstable=false;
     bool Modified=false;
 
     //Loading
-    string Text=TextEdit->toPlainText().toLocal8Bit().data();
-//	if(Text.empty())
-//		return;
-	List.clear();
+    Ztring Text=Ztring().From_UTF8(TextEdit->toPlainText().toUtf8().data());
+
+    if ((!Rules.CodingHistory_Rec || Rules.CodingHistory_Rec_Ex_Comma))
+        AddComma->setChecked(Text.find(__T(",\n"))!=string::npos);
+
+    List.clear();
     List.Separator_Set(0, "\n");
-    if (Text.find(", ")==string::npos)
+    if (Text.find(__T(", "))==string::npos)
         List.Separator_Set(1, ",");
     else
         List.Separator_Set(1, ", ");
     List.Write(Text);
-	int size = List.size();
 
     Table->setRowCount((int)List.size()+1);
     Table->setColumnCount(6);
@@ -463,12 +520,12 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
     Table->setHorizontalHeaderItem(3, new QTableWidgetItem("Word length"));
     Table->setHorizontalHeaderItem(4, new QTableWidgetItem("Mode             "));
     Table->setHorizontalHeaderItem(5, new QTableWidgetItem("Text, free string"));
-    for (size_t Line_Pos=0; Line_Pos<List.size(); Line_Pos++)
+    for (size_t Line_Pos=0; Line_Pos<(size_t)List.size(); Line_Pos++)
     {
         for (size_t Data_Pos=0; Data_Pos<List[Line_Pos].size(); Data_Pos++)
         {
             int Column=-1;
-            Ztring &Value=List[Line_Pos][Data_Pos];
+            string Value=List[Line_Pos][Data_Pos].To_UTF8();
             if (Line_Pos==0 && Data_Pos==0 && Value.find("Coding History ")==0)
             {
                 Modified=true;
@@ -489,16 +546,12 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
             }
             else
                 Ununderstable=true;
-            if (Column==Data_Pos)
+            if (Column==(long)Data_Pos)
             {
+                string Header=Value.substr(0, 2);
                 Value.erase(0, 2);
-                if (Value=="ANALOG")
-                {
-                    Modified=true;
-                    Value="ANALOGUE";
-                }
-                QTableWidgetItem* Item=new QTableWidgetItem(QString().fromUtf8(Value.To_Local().c_str()));
-            
+                QTableWidgetItem* Item=new QTableWidgetItem(QString::fromUtf8(Value.c_str()));
+
                 Table->setItem((int)Line_Pos, Column, Item);
             }
             else if (Column!=-1 || Data_Pos>5)
@@ -513,11 +566,11 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
         }
     }
 
-    Table->setItemDelegateForColumn(0, new CodingAlgorithmDelegate);
-    Table->setItemDelegateForColumn(1, new SamplingFrequencyDelegate);
-    Table->setItemDelegateForColumn(2, new BitRateDelegate);
-    Table->setItemDelegateForColumn(3, new WordLengthDelegate);
-    Table->setItemDelegateForColumn(4, new ModeDelegate);
+    Table->setItemDelegateForColumn(0, new CodingAlgorithmDelegate(Rules));
+    Table->setItemDelegateForColumn(1, new SamplingFrequencyDelegate(Rules));
+    Table->setItemDelegateForColumn(2, new BitRateDelegate(Rules));
+    Table->setItemDelegateForColumn(3, new WordLengthDelegate(Rules));
+    Table->setItemDelegateForColumn(4, new ModeDelegate(Rules));
 
     Table->resizeColumnsToContents();
     Table->resizeRowsToContents();
@@ -533,7 +586,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
                     MessageBox.setStandardButtons(QMessageBox::Close);
                 #endif // (QT_VERSION >= 0x040200)
             MessageBox.setIcon(QMessageBox::Critical);
-            MessageBox.setWindowIcon(QIcon(":/Image/FADGI/Logo.png"));
+            MessageBox.setWindowIcon(QIcon(":/Image/Logo/Logo.png"));
             MessageBox.exec();
         }
 
@@ -560,7 +613,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
             #endif // (QT_VERSION >= 0x040300)
         #endif // (QT_VERSION >= 0x040200)
         MessageBox.setIcon(QMessageBox::Question);
-        MessageBox.setWindowIcon(QIcon(":/Image/FADGI/Logo.png"));
+        MessageBox.setWindowIcon(QIcon(":/Image/Logo/Logo.png"));
         switch (MessageBox.exec())
         {
             case QMessageBox::No    :   //Save was clicked
@@ -573,7 +626,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::Text2List ()
                                         Central->setCurrentIndex(1);
                                         return;
             default                 : ; //Continue
-        } 
+        }
     }
 
     //Preparing
@@ -593,8 +646,11 @@ void GUI_Main_xxxx_CodingHistoryDialog::hideEvent (QHideEvent*)
     if (Central->currentIndex()==0)
         List2Text();
 
+    if (FileName.empty())
+        return;
+
     //Save content
-    std::string Value=TextEdit->toPlainText().toLocal8Bit().data();
+    std::string Value=TextEdit->toPlainText().toUtf8().data();
     if (!C->IsValid(FileName, Field, Value))
     {
         QMessageBox MessageBox;
@@ -604,7 +660,7 @@ void GUI_Main_xxxx_CodingHistoryDialog::hideEvent (QHideEvent*)
             MessageBox.setStandardButtons(QMessageBox::Ok);
         #endif // (QT_VERSION >= 0x040200)
         MessageBox.setIcon(QMessageBox::Warning);
-        MessageBox.setWindowIcon(QIcon(":/Image/FADGI/Logo.png"));
+        MessageBox.setWindowIcon(QIcon(":/Image/Logo/Logo.png"));
         MessageBox.exec();
         return;
     }
