@@ -2,7 +2,7 @@
 //
 // This code was created in 2010 for the Library of Congress and the
 // other federal government agencies participating in the Federal Agencies
-// Digitization Guidelines Initiative and it is in the public domain.
+// Digital Guidelines Initiative and it is in the public domain.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -38,9 +38,9 @@ GUI_Main_xxxx_TimeReferenceDialog::GUI_Main_xxxx_TimeReferenceDialog(Core* _C, c
     Field=Field_;
 
     //Configuration
-    setWindowFlags(windowFlags()&(0xFFFFFFFF-Qt::WindowContextHelpButtonHint));
+    setWindowFlags(windowFlags()&(~Qt::WindowContextHelpButtonHint));
     setWindowTitle("TimeReference");
-    setWindowIcon (QIcon(":/Image/FADGI/Logo.png"));
+    setWindowIcon (QIcon(":/Image/Logo/Logo.png"));
 
     //Buttons
     QDialogButtonBox* Dialog=new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, this);
@@ -56,12 +56,12 @@ GUI_Main_xxxx_TimeReferenceDialog::GUI_Main_xxxx_TimeReferenceDialog(Core* _C, c
     Label->setText(tr("or"));
     LineEdit=new QDoubleSpinBox(this);
     LineEdit->setMinimum(0);
-    LineEdit->setMaximum(0xFFFFFFFF);
+    LineEdit->setMaximum(0xFFFFFFFFFFFFFFFF);
     LineEdit->setDecimals(0);
     connect(LineEdit, SIGNAL(valueChanged (const QString &)), this, SLOT(OnValueEdited (const QString &)));
     LimeEdit_Label=new QLabel(this);
     LimeEdit_Label->setText(tr("(in samples)"));
-        
+
     QGridLayout* L=new QGridLayout();
     L->addWidget(TimeEdit, 0, 0);
     L->addWidget(TimeEdit_Label, 0, 1);
@@ -73,11 +73,11 @@ GUI_Main_xxxx_TimeReferenceDialog::GUI_Main_xxxx_TimeReferenceDialog(Core* _C, c
     setLayout(L);
 
     //Filling
-    SampleRate=Ztring(C->Get(FileName, "SampleRate")).To_int64u();
-    Ztring TimeReference=C->Get(FileName, "TimeReference");
-    Ztring TimeReferenceS=C->Get(FileName, "TimeReference (translated)");
-    TimeEdit->setTime(QTime::fromString(QString().fromUtf8(TimeReferenceS.To_Local().c_str()), Qt::ISODate));
-    LineEdit->setValue(TimeReference.To_int32u());
+    SampleRate=Ztring().From_UTF8(C->Get(FileName, "SampleRate")).To_int64u();
+    Ztring TimeReference=Ztring().From_UTF8(C->Get(FileName, "TimeReference"));
+    Ztring TimeReferenceS=Ztring().From_UTF8(C->Get(FileName, "TimeReference (translated)"));
+    TimeEdit->setTime(QTime::fromString(QString::fromUtf8(TimeReferenceS.To_UTF8().c_str()), Qt::ISODate));
+    LineEdit->setValue(TimeReference.To_int64u());
     IsChanging=false;
 }
 
@@ -88,7 +88,7 @@ GUI_Main_xxxx_TimeReferenceDialog::GUI_Main_xxxx_TimeReferenceDialog(Core* _C, c
 //---------------------------------------------------------------------------
 void GUI_Main_xxxx_TimeReferenceDialog::OnAccept ()
 {
-    std::string Value=LineEdit->text().toLocal8Bit().data();
+    std::string Value=LineEdit->text().toUtf8().data();
     if (!C->IsValid(FileName, Field, Value))
     {
         QMessageBox MessageBox;
@@ -98,7 +98,7 @@ void GUI_Main_xxxx_TimeReferenceDialog::OnAccept ()
             MessageBox.setStandardButtons(QMessageBox::Ok);
         #endif // (QT_VERSION >= 0x040200)
         MessageBox.setIcon(QMessageBox::Warning);
-        MessageBox.setWindowIcon(QIcon(":/Image/FADGI/Logo.png"));
+        MessageBox.setWindowIcon(QIcon(":/Image/Logo/Logo.png"));
         MessageBox.exec();
         return;
     }
@@ -118,7 +118,7 @@ void GUI_Main_xxxx_TimeReferenceDialog::OnTimeChanged (const QTime &Time)
 }
 
 //---------------------------------------------------------------------------
-void GUI_Main_xxxx_TimeReferenceDialog::OnValueEdited (const QString &Value)
+void GUI_Main_xxxx_TimeReferenceDialog::OnValueEdited (const QString&)
 {
     if (IsChanging || SampleRate==0)
         return;
