@@ -2,7 +2,7 @@
 //
 // This code was created in 2010 for the Library of Congress and the
 // other federal government agencies participating in the Federal Agencies
-// Digitization Guidelines Initiative and it is in the public domain.
+// Digital Guidelines Initiative and it is in the public domain.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -10,8 +10,18 @@
 //---------------------------------------------------------------------------
 #include "Riff/Riff_Base.h"
 #include "Riff/Riff_Chunks.h" //Needed for ds64
+#include <algorithm>
 #include <iostream>
+#include <iomanip>
 #include <cstring>
+#include <iomanip>
+
+#ifdef MACSTORE
+#include "Common/Mac_Helpers.h"
+#include "ZenLib/Dir.h"
+#include "Riff_Base.h"
+#endif
+
 using namespace std;
 using namespace ZenLib;
 //---------------------------------------------------------------------------
@@ -54,46 +64,17 @@ void Riff_Base::Read (chunk &Chunk_In)
     //Trace
     if (Chunk.Header.Level)
     {
-        //Offset
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x10000000)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x1000000)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x100000)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x10000)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x1000)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x100)
-            Global->Trace<<'0';
-        if (Global->In.Position_Get()-Chunk.Header.Size<0x10)
-            Global->Trace<<'0';
-        Global->Trace<<Ztring::ToZtring(Global->In.Position_Get()-Chunk.Header.Size, 16)<<' ';
+        Global->Trace<<right<<setfill('0')<<setw(Global->Trace_UseDec?10:8)<<Ztring::ToZtring(Global->In.Position_Get()-Chunk.Header.Size, Global->Trace_UseDec?10:16).To_UTF8()<<' ';
 
         //Size
         int64u Size=Chunk.Content.Size+Chunk.Header.Size-8;
-        if (Size<0x10000000)
-            Global->Trace<<'0';
-        if (Size<0x1000000)
-            Global->Trace<<'0';
-        if (Size<0x100000)
-            Global->Trace<<'0';
-        if (Size<0x10000)
-            Global->Trace<<'0';
-        if (Size<0x1000)
-            Global->Trace<<'0';
-        if (Size<0x100)
-            Global->Trace<<'0';
-        if (Size<0x10)
-            Global->Trace<<'0';
-        Global->Trace<<Ztring::ToZtring(Size, 16)<<' ';
+        Global->Trace<<right<<setfill('0')<<setw(Global->Trace_UseDec?10:8)<<Ztring::ToZtring(Size, Global->Trace_UseDec?10:16).To_UTF8()<<' ';
 
         //Chunk name
         for (size_t Pos=1; Pos<Chunk.Header.Level; Pos++)
             Global->Trace<<"     ";
-        Global->Trace<<Ztring().From_CC4(Chunk.Header.Name);
-        
+        Global->Trace<<Ztring().From_CC4(Chunk.Header.Name).To_UTF8();
+
         //Size
         /*
         for (size_t Pos=Chunk.Header.Level; Pos<4; Pos++)
@@ -103,7 +84,7 @@ void Riff_Base::Read (chunk &Chunk_In)
             Global->Trace<<"+1";
         Global->Trace<<" bytes)";
         */
-        
+
         Global->Trace<<endl;
     }
 
@@ -134,8 +115,14 @@ void Riff_Base::Read (chunk &Chunk_In)
         else
         {
             if (!Global->NoPadding_Accept)
-                throw exception_valid("padding");
-            
+            {
+                ostringstream message;
+                message << Ztring().From_CC4(Chunk.Header.Name).To_UTF8()
+                        << ", missing padding byte at 0x"
+                        << setw(8) << setfill('0') << hex
+                        << Chunk.File_In_Position+Chunk.Header.Size+Chunk.Content.Size+1;
+                throw exception_valid(message.str());
+            }
             //Log
             Global->NoPadding_IsCorrected=true;
 
@@ -161,7 +148,7 @@ void Riff_Base::Read (chunk &Chunk_In)
 bool Riff_Base::Read_Header (chunk &NewChunk)
 {
     int8u Temp[4];
-   
+
     //Chunk name
     if (Global->In.Position_Get()+8>Chunk.File_In_Position+Chunk.Header.Size+Chunk.Content.Size)
         throw exception_valid("small");
@@ -183,16 +170,14 @@ bool Riff_Base::Read_Header (chunk &NewChunk)
     NewChunk.Content.Size=LittleEndian2int32u(Temp);
     if (Global->IsRF64 && Global->ds64 && NewChunk.Header.Level==2 && NewChunk.Header.Name==Elements::WAVE_data && NewChunk.Content.Size==Global->ds64->dataSize%0x100000000LL)
     {
+        Global->RF64DataSize_IsCorrected=true;
         NewChunk.Content.Size=0xFFFFFFFF; //Bug in previous versions, chunk size was not the right one
         NewChunk.Content.IsModified=true; //Forcing write of the write file
         NewChunk.Content.Size_IsModified=true; //Forcing write of the write file
     }
     if (NewChunk.Content.Size==0xFFFFFFFF && Global->IsRF64 && Global->ds64 && NewChunk.Header.Name==Elements::WAVE_data)
         NewChunk.Content.Size=Global->ds64->dataSize; //Setting real WAVE_data size
-    if (Global->In.Position_Get()+NewChunk.Content.Size>Chunk.File_In_Position+Chunk.Header.Size+Chunk.Content.Size
-     && !(NewChunk.Content.Size==0xFFFFFFFF && NewChunk.Header.Name==Elements::RF64)) //Not RF64
-        throw exception_valid("truncated");
-    
+
     //List management (if present)
     if (NewChunk.Header.Name==Elements::LIST || NewChunk.Header.Name==Elements::RIFF || NewChunk.Header.Name==Elements::RF64)
     {
@@ -211,6 +196,19 @@ bool Riff_Base::Read_Header (chunk &NewChunk)
         NewChunk.Header.Size=8;
     }
 
+    if (Global->In.Position_Get()+NewChunk.Content.Size>Global->In.Size_Get() && !(NewChunk.Header.List==Elements::RF64 && NewChunk.Content.Size==0xFFFFFFFB))
+    {
+        if (!Global->TruncatedChunks.str().empty())
+            Global->TruncatedChunks << " ";
+        Global->TruncatedChunks<< "The "
+                               << Ztring().From_CC4(NewChunk.Header.List?NewChunk.Header.List:NewChunk.Header.Name).To_UTF8()
+                               << " chunk should be "
+                               << + NewChunk.Content.Size
+                               << " bytes, but is "
+                               << Global->In.Size_Get()-Global->In.Position_Get()
+                               << " bytes.";
+    }
+
     return true;
 }
 
@@ -223,6 +221,10 @@ void Riff_Base::Read_Internal_ReadAllInBuffer ()
     if (Chunk.Content.Size>(size_t)-1)
         throw exception_read_chunk("non-audio data exceeds available memory");
 
+    //Integrity
+    if (Chunk.File_In_Position+Chunk.Header.Size+Chunk.Content.Size>Global->In.Size_Get())
+        throw exception_valid(!Global->TruncatedChunks.str().empty()?"truncated ("+Global->TruncatedChunks.str()+")":"truncated");
+
     delete Chunk.Content.Buffer; Chunk.Content.Buffer=NULL;
 
     try
@@ -241,7 +243,8 @@ void Riff_Base::Read_Internal_ReadAllInBuffer ()
         if (BytesRead==0)
             break; //Read is finished
         Global->CS.Enter();
-        Global->Progress=(float)Global->In.Position_Get()/Global->In.Size_Get();
+        int64u Skipped=Global->data?Global->data->Size:0;
+        Global->Progress=(float)(Global->In.Position_Get()-Skipped)/Global->In.Size_Get();
         if (Global->Canceling)
         {
             Global->CS.Leave();
@@ -272,13 +275,67 @@ void Riff_Base::Modify (int32u Chunk_Name_1, int32u Chunk_Name_2, int32u Chunk_N
 //---------------------------------------------------------------------------
 void Riff_Base::Modify_Internal_Subs (int32u Chunk_Name_1, int32u Chunk_Name_2, int32u Chunk_Name_3)
 {
+    if (Chunk.Content.IsRemovable)
+        return;
+
+   //Special case: multiples chunks
+   if (Global->adtl && (Chunk_Name_1==Elements::WAVE_adtl_labl
+                     || Chunk_Name_1==Elements::WAVE_adtl_note
+                     || Chunk_Name_1==Elements::WAVE_adtl_ltxt))
+   {
+
+        if (Chunk_Name_1==Elements::WAVE_adtl_labl)
+           Global->adtl->labelsIndex=0;
+        else if (Chunk_Name_1==Elements::WAVE_adtl_note)
+           Global->adtl->notesIndex=0;
+        else if (Chunk_Name_1==Elements::WAVE_adtl_ltxt)
+          Global->adtl->textsIndex=0;
+
+        bool Adding=true;
+        for (size_t Sub_Pos=0; Sub_Pos<Subs.size(); Sub_Pos++)
+        {
+            if (Subs[Sub_Pos]->Chunk.Header.Name!=Chunk_Name_1)
+                continue;
+
+            Subs[Sub_Pos]->Modify(Chunk_Name_2, Chunk_Name_3, 0x00000000);
+            if (Subs[Sub_Pos]->Chunk.Content.IsModified)
+                Chunk.Content.IsModified=true;
+            if (Subs[Sub_Pos]->Chunk.Content.Size_IsModified)
+                Chunk.Content.Size_IsModified=true;
+            if (Subs[Sub_Pos]->Chunk.Content.IsRemovable)
+            {
+                Chunk.Content.IsModified=true;
+                Chunk.Content.Size_IsModified=true;
+                Subs.erase(Subs.begin()+Sub_Pos);
+                Sub_Pos--;
+                Adding=false; //No more data
+            }
+        }
+
+        while(Adding)
+        {
+            size_t Pos=Insert_Internal(Chunk_Name_1);
+            if (Pos>=Subs.size()) //No more data
+            {
+                Adding=false;
+                break;
+            }
+            Chunk.Content.IsModified=true;
+            Chunk.Content.Size_IsModified=true;
+        }
+
+        if (Subs.empty())
+            Chunk.Content.IsRemovable=true;
+
+        return;
+   }
+
     //Parsing subs
-    bool Sub_IsFound=false;
     size_t Sub_Pos=0;
     for (; Sub_Pos<Subs.size(); Sub_Pos++)
         if (Subs[Sub_Pos]->Chunk.Header.Name==Chunk_Name_1)
             break;
-    
+
     //Handling when sub is not present
     if (Sub_Pos>=Subs.size())
     {
@@ -313,12 +370,25 @@ void Riff_Base::Modify_Internal_Subs (int32u Chunk_Name_1, int32u Chunk_Name_2,
 //---------------------------------------------------------------------------
 void Riff_Base::Write ()
 {
+
+    if (Chunk.Header.Level==0)
+    {
+        //Reset buffers and states
+        if(Global->Out_Buffer_Begin.Data)
+            delete[] Global->Out_Buffer_Begin.Data;
+        Global->Out_Buffer_Begin=global::buffer();
+        if(Global->Out_Buffer_End.Data)
+            delete[] Global->Out_Buffer_End.Data;
+        Global->Out_Buffer_End=global::buffer();
+        Global->Out_Buffer_WriteAtEnd=false;
+    }
+
     //Header
     if (Chunk.Header.Level)
     {
         //Calculating block size
         int64u Block_Size=Block_Size_Get();
-        if (Block_Size>RIFF_Size_Limit)
+        if (Block_Size>RIFF_Size_Limit || (Global->IsRF64 && Chunk.Header.Level==1))
         {
             //We need RF64
             if (Chunk.Header.Level==1)
@@ -333,6 +403,8 @@ void Riff_Base::Write ()
                 }
                 if (Chunk.Header.List==Elements::RF64)
                 {
+                    Subs[0]->Modify();
+                    Block_Size=Block_Size_Get();
                     Global->ds64->riffSize=Block_Size-8;
                     if (Global->data)
                         Global->ds64->dataSize=Global->data->Size;
@@ -356,10 +428,10 @@ void Riff_Base::Write ()
         {
             int8u Header[8];
             int32u2BigEndian(Header, Chunk.Header.Name);
-            if (Chunk.Content.Size<=RIFF_Size_Limit)
-                int32u2LittleEndian(Header+4, (int32u)(Chunk.Content.Size));
-            else if (Chunk.Header.Level==2 && Chunk.Header.Name==Elements::WAVE_data)
+            if ((Global->IsRF64 || Chunk.Content.Size>RIFF_Size_Limit) && Global->ds64 && Chunk.Header.Level==2 && Chunk.Header.Name==Elements::WAVE_data)
                 int32u2LittleEndian(Header+4, 0xFFFFFFFF);
+            else if (Chunk.Content.Size<=RIFF_Size_Limit)
+                int32u2LittleEndian(Header+4, (int32u)(Chunk.Content.Size));
             else
                 throw exception_write("Block size is too big");
             Write_Internal(Header, 8);
@@ -415,7 +487,7 @@ void Riff_Base::Write ()
         }
         while (DataChunkMustBeMoved);
     }
-    
+
     //Content
     if (Chunk.Content.IsModified)
     {
@@ -425,7 +497,7 @@ void Riff_Base::Write ()
         else
         {
             Write_Internal();
-            
+
             //Padding
             if (Chunk.Content.Size%2)
             {
@@ -438,7 +510,7 @@ void Riff_Base::Write ()
     else
     {
         Write_Internal();
-        
+
         //Padding
         if (Chunk.Content.Size%2)
         {
@@ -491,7 +563,14 @@ void Riff_Base::Write ()
         else
         {
             //Real writing
-            if (!Global->Out.Create(Global->File_Name+".tmp", false))
+            #ifdef MACSTORE
+            Global->Temp_Name=makeUniqueFileName();
+            Global->Temp_Path=makeTemporaryDirectoryForFile(Global->File_Name);
+
+            if (!Global->Out.Create(Global->Temp_Path+Global->Temp_Name, false))
+            #else
+            if (!Global->Out.Create(Global->File_Name+__T(".tmp"), false))
+            #endif
                 throw exception_write("Can not create temporary file");
 
             //Begin
@@ -527,8 +606,20 @@ void Riff_Base::Write ()
             //Renaming files
             if (!File::Delete(Global->File_Name))
                 throw exception_write("Original file can't be deleted");
-            if (!File::Move(Global->File_Name+".tmp", Global->File_Name))
+            #ifdef MACSTORE
+            if (!File::Move(Global->Temp_Path+Global->Temp_Name, Global->File_Name))
+            #else
+            if (!File::Move(Global->File_Name+__T(".tmp"), Global->File_Name))
+            #endif
                 throw exception_write("Temporary file can't be renamed");
+
+            #ifdef MACSTORE
+            if (Global->Temp_Path.size() && Dir::Exists(Global->Temp_Path))
+                deleteTemporaryDirectory(Global->Temp_Path);
+
+            Global->Temp_Name=__T("");
+            Global->Temp_Path=__T("");
+            #endif
         }
     }
 }
@@ -552,9 +643,9 @@ void Riff_Base::Read_Internal ()
         default                  :
                                     if (!Global->UnsupportedChunks.empty())
                                         Global->UnsupportedChunks+=" ";
-                                    Global->UnsupportedChunks+=Ztring().From_CC4(Chunk.Header.Name);
+                                    Global->UnsupportedChunks+=Ztring().From_CC4(Chunk.Header.Name).To_UTF8();
     }
-        
+
     Read_Internal_ReadAllInBuffer();
 }
 
@@ -562,7 +653,7 @@ void Riff_Base::Read_Internal ()
 void Riff_Base::Write_Internal ()
 {
     Global->In.GoTo(Chunk.File_In_Position+Chunk.Header.Size);
-    
+
     int8u Temp[32768];
     while(Global->In.Position_Get()<Chunk.File_In_Position+Chunk.Header.Size+Chunk.Content.Size)
     {
@@ -646,8 +737,15 @@ void Riff_Base::Write_Internal (const int8u* Temp, size_t Temp_Offset)
         {
             if (!Global->Out.Opened_Get())
             {
-                if (!Global->Out.Create(Global->File_Name+".tmp"))
-                    throw exception_write(Global->File_Name+": temporary file can not be created");
+                #ifdef MACSTORE
+                Global->Temp_Name=makeUniqueFileName();
+                Global->Temp_Path=makeTemporaryDirectoryForFile(Global->File_Name);
+                if (!Global->Out.Create(Global->Temp_Path+Global->Temp_Name))
+                    throw exception_write(Ztring(Global->Temp_Path+Global->Temp_Name).To_UTF8()+": temporary file can not be created");
+                #else
+                if (!Global->Out.Create(Global->File_Name+__T(".tmp")))
+                    throw exception_write(Global->File_Name.To_UTF8()+": temporary file can not be created");
+                #endif
             }
             if (Global->Out.Write(Temp, Temp_Offset)<Temp_Offset)
                 throw exception_write();
@@ -673,7 +771,7 @@ int64u Riff_Base::Block_Size_Get ()
         return (Chunk.Header.List==0x00000000?8:12)+((Chunk.Content.Size%2)?(Chunk.Content.Size+1):Chunk.Content.Size);
     if (Subs.empty())
         return 8+((Chunk.Content.Size%2)?(Chunk.Content.Size+1):Chunk.Content.Size);
-    
+
     //Parsing subs
     int64u Size=0;
     for (size_t Pos=0; Pos<Subs.size(); Pos++)
